/*
    Text maid
    copyright (c) 1998-2002 Iwamoto,Kazuki http://www.maid.org/ iwm@maid.org

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "print.h"
#ifdef USE_GDKPIXBUF
#include <gdk/gdkkeysyms.h>
#include <gdk-pixbuf/gdk-pixbuf.h>
#include <stdio.h>
#include "abort.h"
#include "charset.h"
#include "edit.h"
#include "other.h"


PRNCFG printer;						/*  */


/******************************************************************************
*                                                                             *
* ؿ                                                        *
*                                                                             *
******************************************************************************/
static gboolean print_clicked_ok;
static GtkWidget *dialog,*button0,*button1,*check,*entry0,*entry1,*radio0;
static GtkWidget *label1,*label2,*label3;
static GtkWidget *spin0,*spin1,*spin2,*spin3,*spin4,*spin5,*spin6;
static struct {
	gchar *name;
	gint width,height;
} paper[]={	{"A3",840,1188},{"A4",594,840},{"A5",420,594},
			{"B4",730,1030},{"B5",515,730},{NULL,0,0}};


/* OKܥɽ/ɽȽ */
static void print_dialog_ok_sensitive(GtkWidget *widget,PRNCFG *prncfg)
{
	gint column,row,width,height,left,top,right,bottom;

	column=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin0));
	width=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin1));
	height=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin2));
	top=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin3));
	left=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin4));
	right=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin5));
	bottom=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin6));
	width-=left+right;
	height-=top+bottom;
	row=left>=0 && top>=0 && right>=0 && bottom>=0 && width>0 && height>0
				?column*prncfg->fontwidth*height/(width*prncfg->fontheight):0;
	gtk_widget_set_sensitive(button1,column>=1 && row>=1
		&& strlen(gtk_entry_get_text(GTK_ENTRY(gtk_toggle_button_get_active(
								GTK_TOGGLE_BUTTON(radio0))?entry0:entry1)))>0);
}


/* ԥܥѲ */
static void print_dialog_changed_spin(GtkWidget *widget,PRNCFG *prncfg)
{
	gchar *text;
	gint column,row,width,height,left,top,right,bottom;

	column=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin0));
	width=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin1));
	height=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin2));
	top=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin3));
	left=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin4));
	right=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin5));
	bottom=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin6));
	width-=left+right;
	height-=top+bottom;
	row=left>=0 && top>=0 && right>=0 && bottom>=0 && width>0 && height>0
				?column*prncfg->fontwidth*height/(width*prncfg->fontheight):0;
	text=g_strdup_printf(_("%d"),MAX(row,0));
	gtk_label_set_text(GTK_LABEL(label1),text);
	g_free(text);
	print_dialog_ok_sensitive(widget,prncfg);
}


/* ܥܥå */
static void print_dialog_changed_combo(GtkWidget *widget,gpointer user_data)
{
	gchar *name;
	gint i;

	name=gtk_entry_get_text(GTK_ENTRY(widget));
	for (i=0;paper[i].name!=NULL;i++)
		if (strcmp(paper[i].name,name)==0)
			break;
	if (paper[i].name==NULL) {
		/* 桼 */
		gtk_widget_set_sensitive(label2,TRUE);
		gtk_widget_set_sensitive(label3,TRUE);
		gtk_widget_set_sensitive(spin1,TRUE);
		gtk_widget_set_sensitive(spin2,TRUE);
	} else {
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin1),paper[i].width);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin2),paper[i].height);
		gtk_widget_set_sensitive(label2,FALSE);
		gtk_widget_set_sensitive(label3,FALSE);
		gtk_widget_set_sensitive(spin1,FALSE);
		gtk_widget_set_sensitive(spin2,FALSE);
	}
}


/* åܥ/饸ܥ */
static void print_dialog_toggled(GtkWidget *widget,PRNCFG *prncfg)
{
	gboolean result;

	result=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio0));
	gtk_widget_set_sensitive(entry0,result);
	gtk_widget_set_sensitive(entry1,!result);
	gtk_widget_set_sensitive(button0,!result);
	print_dialog_ok_sensitive(widget,prncfg);
}


/* OKܥ󤬲줿 */
static void print_dialog_clicked_ok(GtkWidget *widget,PRNCFG *prncfg)
{
	prncfg->mode=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio0));
	if (prncfg->mode) {
		g_free(prncfg->printer);
		prncfg->printer=g_strdup(gtk_entry_get_text(GTK_ENTRY(entry0)));
	} else {
		g_free(prncfg->file);
		prncfg->file=g_strdup(gtk_entry_get_text(GTK_ENTRY(entry1)));
	}
	prncfg->select=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
	prncfg->column=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin0));
	prncfg->width=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin1));
	prncfg->height=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin2));
	prncfg->top=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin3));
	prncfg->left=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin4));
	prncfg->right=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin5));
	prncfg->bottom=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin6));
    gtk_widget_destroy(dialog);
	print_clicked_ok=TRUE;
}


static gchar *print_open_file;


/* եλȤOKܥ󤬲줿 */
static void print_open_ok(GtkWidget *widget,GtkWidget *dialog_open)
{
	print_open_file=g_strdup(gtk_file_selection_get_filename(
											GTK_FILE_SELECTION(dialog_open)));
    gtk_widget_destroy(dialog_open);
}


/* ȥܥ󤬲줿 */
static void print_dialog_clicked_file(GtkWidget *widget,PRNCFG *prncfg)
{
	GtkWidget *dialog_open;

	print_open_file=NULL;
	dialog_open=gtk_file_selection_new(_("̾դ¸"));
	gtk_signal_connect(GTK_OBJECT(dialog_open),"destroy",gtk_main_quit,NULL);
	gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(dialog_open)->ok_button),
										"clicked",print_open_ok,dialog_open);
	gtk_signal_connect_object(
					GTK_OBJECT(GTK_FILE_SELECTION(dialog_open)->cancel_button),
						"clicked",gtk_widget_destroy,GTK_OBJECT(dialog_open));
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(dialog_open),
										gtk_entry_get_text(GTK_ENTRY(entry1)));
	gtk_widget_show(dialog_open);
	gtk_grab_add(dialog_open);
	gtk_main();
	if (print_open_file!=NULL) {
		gtk_entry_set_text(GTK_ENTRY(entry1),print_open_file);
		g_free(print_open_file);
	}
}


/* ȥ꡼/ԥܥǥ꥿󤬲줿 */
static void print_dialog_activate(GtkWidget *widget,PRNCFG *prncfg)
{
	gint left,top,right,bottom;

	top=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin3));
	left=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin4));
	right=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin5));
	bottom=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin6));
	if (strlen(gtk_entry_get_text(GTK_ENTRY(gtk_toggle_button_get_active(
								GTK_TOGGLE_BUTTON(radio0))?entry0:entry1)))>0
		&& gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin0))>=2
		&& left>=0 && top>=0 && right>=0 && bottom>=0
		&& gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin1))>left+right
		&& gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin2))>top+bottom)
		print_dialog_clicked_ok(widget,prncfg);
}


/* ESC줿 */
static gboolean print_dialog_key_press(GtkWidget *widget,GdkEventKey *event,
															gpointer user_data)
{
	if (event->keyval==GDK_Escape)
		gtk_widget_destroy(widget);
	return FALSE;
}


gboolean print_dialog(PRNCFG *prncfg)
{
	gint i;
	guint key;
	GList *glist=NULL;
	GtkAccelGroup *accel;
	GtkWidget *button2,*combo,*radio1;
	GtkWidget *frame0,*frame1,*hbox0,*hbox1,*vbox;
	GtkWidget *label0,*label4,*label5,*label6,*label7;

	/* ᥤ󥦥ɥ */
	dialog=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_set_title(GTK_WINDOW(dialog),_(""));
	gtk_signal_connect_after(GTK_OBJECT(dialog),"key-press-event",
								GTK_SIGNAL_FUNC(print_dialog_key_press),NULL);
	gtk_signal_connect(GTK_OBJECT(dialog),"destroy",gtk_main_quit,NULL);
	/* 졼 */
	accel=gtk_accel_group_new();
	gtk_accel_group_attach(accel,GTK_OBJECT(dialog));
	/* ܥ */
	button0=gtk_button_new_with_label(_("..."));
	button1=gtk_button_new_with_label(_(""));
	button2=gtk_button_new_with_label(_(""));
	gtk_signal_connect(GTK_OBJECT(button0),"clicked",
											print_dialog_clicked_file,dialog);
	gtk_signal_connect(GTK_OBJECT(button1),"clicked",
											print_dialog_clicked_ok,prncfg);
	gtk_signal_connect_object(GTK_OBJECT(button2),"clicked",
										gtk_widget_destroy,GTK_OBJECT(dialog));
	GTK_WIDGET_SET_FLAGS(button0,GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS(button1,GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS(button2,GTK_CAN_DEFAULT);
	/* åܥ */
	check=gtk_check_button_new_with_label(NULL);
	key=gtk_label_parse_uline(GTK_LABEL(GTK_BIN(check)->child),
													_("ϰϤ(_S)"));
	gtk_widget_add_accelerator(check,"clicked",accel,key,GDK_MOD1_MASK,0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),prncfg->select);
	gtk_signal_connect(GTK_OBJECT(check),"toggled",
												print_dialog_toggled,prncfg);
	gtk_widget_set_sensitive(check,prncfg->select);
	/* ȥ꡼ */
	entry0=gtk_entry_new();
	entry1=gtk_entry_new();
	if (prncfg->printer!=NULL)
		gtk_entry_set_text(GTK_ENTRY(entry0),prncfg->printer);
	if (prncfg->file!=NULL)
		gtk_entry_set_text(GTK_ENTRY(entry1),prncfg->file);
	gtk_signal_connect(GTK_OBJECT(entry0),"activate",
											print_dialog_clicked_ok,prncfg);
	gtk_signal_connect(GTK_OBJECT(entry1),"activate",
											print_dialog_clicked_ok,prncfg);
	gtk_signal_connect(GTK_OBJECT(entry0),"changed",
											print_dialog_ok_sensitive,prncfg);
	gtk_signal_connect(GTK_OBJECT(entry1),"changed",
											print_dialog_ok_sensitive,prncfg);
	/* ԥܥ */
	spin0=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->column,
													1,G_MAXINT,1,5,0)),0,0);
	spin1=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->width,
													1,G_MAXINT,1,5,0)),0,0);
	spin2=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->height,
													1,G_MAXINT,1,5,0)),0,0);
	spin3=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->top,
													0,G_MAXINT,1,5,0)),0,0);
	spin4=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->left,
													0,G_MAXINT,1,5,0)),0,0);
	spin5=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->right,
													0,G_MAXINT,1,5,0)),0,0);
	spin6=gtk_spin_button_new(GTK_ADJUSTMENT(gtk_adjustment_new(prncfg->bottom,
													0,G_MAXINT,1,5,0)),0,0);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin0),TRUE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin1),TRUE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin2),TRUE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin3),TRUE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin4),TRUE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin5),TRUE);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin6),TRUE);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin0),GTK_SHADOW_IN);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin1),GTK_SHADOW_IN);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin2),GTK_SHADOW_IN);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin3),GTK_SHADOW_IN);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin4),GTK_SHADOW_IN);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin5),GTK_SHADOW_IN);
	gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin6),GTK_SHADOW_IN);
	gtk_signal_connect(GTK_OBJECT(spin0),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin1),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin2),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin3),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin4),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin5),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin6),"activate",
												print_dialog_activate,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin0),"changed",
											print_dialog_changed_spin,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin1),"changed",
											print_dialog_changed_spin,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin2),"changed",
											print_dialog_changed_spin,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin3),"changed",
											print_dialog_changed_spin,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin4),"changed",
											print_dialog_changed_spin,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin5),"changed",
											print_dialog_changed_spin,prncfg);
	gtk_signal_connect(GTK_OBJECT(spin6),"changed",
											print_dialog_changed_spin,prncfg);
	/* 饸ܥ */
	radio0=gtk_radio_button_new_with_label(NULL,NULL);
	radio1=gtk_radio_button_new_with_label_from_widget(
												GTK_RADIO_BUTTON(radio0),NULL);
	key=gtk_label_parse_uline(GTK_LABEL(GTK_BIN(radio0)->child),
															_("ץ(_P)"));
	gtk_widget_add_accelerator(radio0,"clicked",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(GTK_BIN(radio1)->child),
															_("ե(_F)"));
	gtk_widget_add_accelerator(radio1,"clicked",accel,key,GDK_MOD1_MASK,0);
	gtk_signal_connect(GTK_OBJECT(radio0),"toggled",
												print_dialog_toggled,prncfg);
	gtk_signal_connect(GTK_OBJECT(radio1),"toggled",
												print_dialog_toggled,prncfg);
	if (!prncfg->mode)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1),TRUE);
	/* ٥ */
	label0=gtk_label_new(NULL);
	label1=gtk_label_new(NULL);
	label2=gtk_label_new(NULL);
	label3=gtk_label_new(NULL);
	label4=gtk_label_new(NULL);
	label5=gtk_label_new(NULL);
	label6=gtk_label_new(NULL);
	label7=gtk_label_new(NULL);
	key=gtk_label_parse_uline(GTK_LABEL(label0),_("(_C)"));
	gtk_widget_add_accelerator(spin0,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(label2),_("(_W)"));
	gtk_widget_add_accelerator(spin1,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(label3),_("⤵(_H)"));
	gtk_widget_add_accelerator(spin2,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(label4),_("(_T)"));
	gtk_widget_add_accelerator(spin3,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(label5),_("(_L)"));
	gtk_widget_add_accelerator(spin4,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(label6),_("(_R)"));
	gtk_widget_add_accelerator(spin5,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	key=gtk_label_parse_uline(GTK_LABEL(label7),_("(_B)"));
	gtk_widget_add_accelerator(spin6,"grab-focus",accel,key,GDK_MOD1_MASK,0);
	/* ܥܥå */
	for (i=0;paper[i].name!=NULL;i++)
		glist=g_list_append(glist,paper[i].name);
	glist=g_list_append(glist,_("桼"));
	combo=gtk_combo_new();
	gtk_combo_set_popdown_strings(GTK_COMBO(combo),glist);
	gtk_combo_set_value_in_list(GTK_COMBO(combo),TRUE,FALSE);
	gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(combo)->entry),FALSE);
	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(combo)->entry),"changed",
											print_dialog_changed_combo,NULL);
	for (i=0;paper[i].name!=NULL;i++)
		if (paper[i].width==prncfg->width && paper[i].height==prncfg->height)
			break;
	if (paper[i].name==NULL) {
	} else {
		gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(combo)->entry),paper[i].name);
	}

	/* ե졼ȥܥå */
	frame0=gtk_frame_new(_("ѻ"));
	vbox=gtk_vbox_new(FALSE,8);
	gtk_container_set_border_width(GTK_CONTAINER(vbox),8);
	gtk_box_pack_start(GTK_BOX(vbox),combo,TRUE,TRUE,0);
	hbox0=gtk_hbox_new(FALSE,8);
	hbox1=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),label2,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),spin1,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),hbox1,TRUE,TRUE,0);
	hbox1=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),label3,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),spin2,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),hbox1,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);
	gtk_container_add(GTK_CONTAINER(frame0),vbox);

	frame1=gtk_frame_new(_("ޡ"));
	vbox=gtk_vbox_new(FALSE,8);
	gtk_container_set_border_width(GTK_CONTAINER(vbox),8);
	hbox0=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),label4,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),spin3,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);
	hbox0=gtk_hbox_new(FALSE,8);
	hbox1=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),label5,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),spin4,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),hbox1,TRUE,TRUE,0);
	hbox1=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),label6,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),spin5,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),hbox1,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);
	hbox0=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),label7,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),spin6,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);
	gtk_container_add(GTK_CONTAINER(frame1),vbox);

	vbox=gtk_vbox_new(FALSE,8);
	gtk_container_set_border_width(GTK_CONTAINER(vbox),8);

	hbox0=gtk_hbox_new(FALSE,8);
	hbox1=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),spin0,TRUE,TRUE,0);
	gtk_box_pack_end(GTK_BOX(hbox1),label0,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),hbox1,TRUE,TRUE,0);
	gtk_box_pack_end(GTK_BOX(hbox0),label1,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);

	gtk_box_pack_start(GTK_BOX(vbox),check,FALSE,FALSE,0);

	hbox0=gtk_hbox_new(FALSE,8);
	gtk_box_pack_start(GTK_BOX(hbox0),radio0,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),entry0,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);

	hbox0=gtk_hbox_new(FALSE,8);
	gtk_box_pack_start(GTK_BOX(hbox0),radio1,FALSE,FALSE,0);
	hbox1=gtk_hbox_new(FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),entry1,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(hbox1),button0,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(hbox0),hbox1,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);

	gtk_box_pack_start(GTK_BOX(vbox),frame0,FALSE,FALSE,0);
	gtk_box_pack_start(GTK_BOX(vbox),frame1,FALSE,FALSE,0);

	hbox0=gtk_hbox_new(FALSE,8);
	gtk_box_pack_end(GTK_BOX(hbox0),button2,FALSE,FALSE,0);
	gtk_box_pack_end(GTK_BOX(hbox0),button1,FALSE,FALSE,0);
	gtk_box_pack_end(GTK_BOX(vbox),hbox0,FALSE,FALSE,0);

	gtk_container_add(GTK_CONTAINER(dialog),vbox);

	/* ɽ */
	gtk_widget_set_sensitive(entry0,prncfg->mode);
	gtk_widget_set_sensitive(entry1,!prncfg->mode);
	gtk_widget_set_sensitive(button0,!prncfg->mode);
	gtk_widget_grab_focus(button1);
	print_dialog_changed_spin(NULL,prncfg);
	print_clicked_ok=FALSE;

	gtk_window_set_policy(GTK_WINDOW(dialog),FALSE,TRUE,FALSE);
	gtk_grab_add(dialog);
	gtk_widget_show_all(dialog);
	gtk_main();

	return print_clicked_ok;
}


/******************************************************************************
*                                                                             *
* ؿ                                                                  *
*                                                                             *
******************************************************************************/
gboolean print_out(TEXTWND *ptw,PRNCFG *prncfg)
{
	guchar *c;
	gint i,x,y,sx,sy,datapos,endpos,line,width,channels,rowstride;
	FILE *fp;
	LINEBUF *p;
	GdkGC *gc;
	GdkPixmap *pixmap;
	GdkPixbuf *pixbuf;
	GdkPoint start,end;

	sx=prncfg->column*prncfg->fontwidth;
	sy=sx*(prncfg->height-prncfg->left-prncfg->right)
								/(prncfg->width-prncfg->top-prncfg->bottom);
	if (ptw->select.x<0 || !prncfg->select) {
		/* ϰϤʤ٤ư */
		start.x=start.y=0;
		end.x=get_width(ptw,ptw->max-1);
		end.y=ptw->max-1;
	} else if (ptw->select.y<ptw->cursor.y
			|| (ptw->select.y==ptw->cursor.y && ptw->select.x<ptw->cursor.x)) {
		start=ptw->select;
		end=ptw->cursor;
	} else {
		start=ptw->cursor;
		end=ptw->select;
	}
	if (prncfg->mode) {
		/* ץ */
		if ((fp=popen(prncfg->printer,"w"))==NULL) {
			message_box("Text maid",_("ץ󥿤ޤ"),0,
															_("λ"),NULL);
			return FALSE;
		}
	} else {
		/* ե */
		if ((fp=fopen(prncfg->file,"wt"))==NULL) {
			message_box("Text maid",_("ե뤬ޤ"),0,
															_("λ"),NULL);
			return FALSE;
		}
	}
	fputs("%!PS-Adobe-3.0\n",fp);
	pixmap=gdk_pixmap_new(window->window,sx,sy,-1);
	pixbuf=gdk_pixbuf_new(GDK_COLORSPACE_RGB,FALSE,8,sx,sy);
	gc=gdk_gc_new(pixmap);
	gdk_gc_set_line_attributes(gc,1,
								GDK_LINE_SOLID,GDK_CAP_BUTT,GDK_JOIN_MITER);
	for (i=10;i<12;i++)
		gdk_color_alloc(gdk_colormap_get_system(),
									(ptw->syscol?system_color:ptw->color)+i);
	p=get_line_buf(&ptw->start,&ptw->off,start.y);
	endpos=get_data_pos(ptw,end.x,end.y,FALSE);
	line=start.y;
	datapos=get_data_pos(ptw,start.x,start.y,FALSE);
	/* ܥå */
	userbreak=TRUE;
	dialog=abort_dialog(_(""));
	gtk_grab_add(dialog);
	while (line<=end.y) {
		/* ڡ */
		/* ط */
		gdk_gc_set_foreground(gc,(ptw->syscol?system_color:ptw->color)+11);
		gdk_draw_rectangle(pixmap,gc,TRUE,0,0,sx,sy);
		gdk_gc_set_foreground(gc,(ptw->syscol?system_color:ptw->color)+10);
		x=y=0;
		while (y<=sy-ptw->fontheight && line<=end.y) {
			while (gtk_events_pending())
				gtk_main_iteration();
			while (x<sx) {
				if (datapos>=(line==end.y?endpos:p->length)) {
					/* Ԥν */
					datapos=0;
					line++;
					p=p->next;
					if (p==NULL || !p->margin)
						break;
				}
				if (p->text[datapos]=='\t') {
					/*  */
					width=(x/ptw->fontwidth/ptw->tab+1)*ptw->tab
															*ptw->fontwidth-x;
					if (sx<width+x)
						break;
					datapos++;
				} else if (datapos+charset_length(p->text[datapos])
																<=p->length) {
					/* Ⱦ/ */
					width=ptw->fontwidth*charset_width(ptw->font,
											p->text+datapos,ptw->fontwidth);
					if (sx<x+width)
						break;
					/* ʸ */
					gdk_draw_text(pixmap,ptw->font,gc,
										x,y+ptw->fontascent,p->text+datapos,
											charset_length(p->text[datapos]));
					datapos+=charset_length(p->text[datapos]);
				} else {
					/* Ⱦ */
					width=ptw->fontwidth;
					if (sx<x+width)
						break;
					gdk_draw_text(pixmap,ptw->font,gc,
										x,y+ptw->fontascent,p->text+datapos,1);
					datapos++;
				}
				x+=width;
			}
			x=0;
			y+=ptw->fontheight;
		}
		fprintf(fp,
			"%d %d translate\n"
			"%d %d scale\n"
			"%d %d 8 [%d 0 0 %d 0 0]\n"
			"{currentfile 32 string readhexstring pop}\n"
			"image\n",
			prncfg->left,prncfg->bottom,
			prncfg->width-prncfg->left-prncfg->right,
			prncfg->height-prncfg->top-prncfg->bottom,
			sx,sy,sx,sy);
		gdk_pixbuf_get_from_drawable(pixbuf,pixmap,gdk_colormap_get_system(),
																0,0,0,0,sx,sy);
		channels=gdk_pixbuf_get_n_channels(pixbuf);
		rowstride=gdk_pixbuf_get_rowstride(pixbuf);
		c=gdk_pixbuf_get_pixels(pixbuf);
		i=0;
		switch (channels) {
			case 1:
				for (y=sy-1;y>=0;y--)
					for (x=0;x<sx;x++) {
						fprintf(fp,"%02x",c[x*channels+y*rowstride]);
						if (++i%32==0)
							fputc('\n',fp);
					}
				break;
			case 3:
			case 4:
				for (y=sy-1;y>=0;y--)
					for (x=0;x<sx;x++) {
						fprintf(fp,"%02x",(c[x*channels+y*rowstride]*77
										+c[x*channels+y*rowstride+1]*150
										+c[x*channels+y*rowstride+2]*29)/256);
						if (++i%32==0)
							fputc('\n',fp);
					}
				break;
			default:
				for (y=0;y<sy;y++)
					for (x=0;x<sx;x++) {
						fputs("ff",fp);
						if (++i%32==0)
							fputc('\n',fp);
					}
		}
		fputs("showpage\n",fp);
		if (!userbreak)
			goto loop;
	}
	loop:
	/* λ */
	gtk_grab_remove(dialog);
    gtk_widget_destroy(dialog);
	gdk_gc_destroy(gc);
	gdk_pixbuf_unref(pixbuf);
	gdk_pixmap_unref(pixmap);
	if (prncfg->mode)	/* ץ */
		pclose(fp);
	else				/* ե */
		fclose(fp);
	return TRUE;
}


#endif
